/*
    ===============================================================

      ORBIS.H - Header das rotinas para interface com C e C++

                  ORBIS - Biblioteca Universal de Rotinas

    Copyright  (c)1992-94 AG SISTEMAS DE HARDWARE E SOFTWARE S/C Ltda.
                      Todos os direitos reservados
    ================================================================

    Defina as seguintes constantes para inibir a geracao dos prototipos
    correspondentes:

     SEM_GRAFICOS
     SEM_COMUNICACAO
     SEM_DOS
     SEM_IMPRESSAO
     SEM_COMPRESSAO
     SEM_TELA
     SEM_ACENTUACAO
     SEM_MOUSE
     SEM_XMS
     SEM_MULTIMIDIA
     SEM_REDE
*/
#ifndef _INC_ORBIS
#define _INC_ORBIS

#pragma pack(1)         /* Assume alinhamento em bytes */

#if defined( __cplusplus )
extern "C" {
#endif  /* __cplusplus */


/**** Compatibilidade com WATCOM C ******/
#ifdef __WATCOMC__
   #pragma aux __pascal "^" parm reverse routine [] \
              value struct float struct caller [] \
              modify [ax bx cx dx es si di] loadds
   #define PASCAL __pascal
#else
   #ifndef PASCAL
   #define PASCAL pascal
   #endif
#endif

#ifndef FAR
#define FAR far
#endif

#define ORBISAPI               FAR PASCAL



/********** Tipos de dados e variveis externas *******/

typedef int  FAR *             LPINT;
typedef char FAR *             LPSTR;
typedef unsigned char FAR *    LPBYTE;
typedef void FAR *             LPVOID;

typedef unsigned               WORD16;
typedef long                   WORD32;


#ifndef SEM_GRAFICOS
/********************** Rotinas Grficas *************************/

#define adaptador_grafico MODOVIDEO   /* Compatibilidade com versoes anteriores */
#define stFonteVideo      FONTEVIDEO  /* Compatibilidade com versoes anteriores */

typedef enum {
   NENHUM   = 0x00,
   CGA      = 0x06,     /* CGA,        640x200, 2 cores, 0xB800 */
   HERCULES = 0x07,     /* HERCULES,   720x348, 2 cores, 0xB000 */
   EGA_MONO = 0x0F,     /* EGA,        640x350, 2 cores, 0xA000 */
   EGA_COLOR= 0x10,     /* EGA,        640x350, 16 cores, 0xA000 */
   VGA_MONO = 0x11,     /* VGA,        640x480, 2 cores, 0xA000 */
   VGA_COLOR= 0x12,     /* VGA,        640x480, 16 cores, 0xA000 */
   VGA_COLOR2=0x13,     /* VGA,        320x200, 256 cores, 0xA000 */
                             /* Modos SVGA - apenas em BIOS VESA */
   SVGA_640x480x256=0x101,
   SVGA_800x600x16 =0x102,
   SVGA_800x600x256=0x103,
   SVGA_1024x768x16=0x104,
   SVGA_1024x768x256=0x105,
   SVGA_1280x1024x16=0x106,
   SVGA_1280x1024x256=0x107
   } MODOVIDEO;

typedef struct
   {
   int       iAltura;
   struct
      {
      int    iCharWidth;
      int    iCharHeight;
      int    iDeltaX;
      int    iTopOffset;
      int    iLeftOffset;
      LPBYTE lpBitMap;
      } Caractere[256];
   } FONTEVIDEO;

extern MODOVIDEO FAR Gr_Adaptor;
extern int FAR Gr_Char_W;
extern int FAR Gr_Char_H;
extern int FAR Gr_CorFrente;
extern int FAR Gr_CorFundo;
extern int FAR Gr_ArcXi;
extern int FAR Gr_ArcYi;
extern int FAR Gr_ArcXf;
extern int FAR Gr_ArcYf;
extern int FAR Gr_Sobreescreve;
extern int FAR Gr_PixelsH;
extern int FAR Gr_PixelsV;

extern unsigned FAR Gr_LineStyle;
extern unsigned FAR Gr_TelaSeg;
extern unsigned FAR Gr_Colors;

MODOVIDEO ORBISAPI Inicia_Grafico (MODOVIDEO Tipo);
void     ORBISAPI Fim_Grafico (void);
void     ORBISAPI __putpixel (unsigned X, unsigned Y, int COLOR);
int      ORBISAPI __getpixel (unsigned X, unsigned Y);
int      ORBISAPI getmaxcolor (void);
int      ORBISAPI getmaxx (void);
int      ORBISAPI getmaxy (void);
int      ORBISAPI getx (void);
int      ORBISAPI gety (void);
void     ORBISAPI moveto (int X, int Y);
void     ORBISAPI moverel (int dX, int dY);
void     ORBISAPI _set_color (int COR);
int      ORBISAPI _getcolor (void);
void     ORBISAPI setbkcolor (int COR);
void     ORBISAPI line (int Xi, int Yi, int Xf, int Yf);
void     ORBISAPI floodline (int Xi, int Yi, int Xf, int Yf);
void     ORBISAPI linerel (int dX, int dY);
void     ORBISAPI lineto (int X, int Y);
void     ORBISAPI floodlineto (int X, int Y);
void     ORBISAPI rectangle (int Xi, int Yi, int Xf, int Yf);
void     ORBISAPI setlinestyle (unsigned Estilo);
void     ORBISAPI floodputpixel (int X, int Y);
void     ORBISAPI floodfill (int Xp, int Yp, int BORDA);
void     ORBISAPI userfillstyle (LPBYTE Padrao);
void     ORBISAPI setfillstyle (int Padrao);
void     ORBISAPI bar (int Xi, int Yi, int Xf, int Yf);
int      ORBISAPI sin (unsigned Degrees);
int      ORBISAPI cos (unsigned Degrees);
void     ORBISAPI arc (int Xc, int Yc, int R, int ANG_INI, int ANG_FIM);
void     ORBISAPI pieslice (int Xc, int Yc, int R,
                              int ANG_INI, int ANG_FIM, int FILLCOLOR);
void     ORBISAPI circle (int Xc, int Yc, int R);
void     ORBISAPI setaspect (unsigned Aspecto);
unsigned ORBISAPI getaspect (void);
void     ORBISAPI ggotoxy (int col, int lin);
void     ORBISAPI OutChar (unsigned char);
void     ORBISAPI tograf (void);
int      ORBISAPI ShowPCX (LPSTR Arquivo, int X1, int Y1,
                             int X2, int Y2, int MODO);
int      ORBISAPI GetPCXSettings (LPSTR Arquivo, LPINT TamX, LPINT TamY);
int      ORBISAPI ShowBMP (LPSTR Arquivo, int X1, int Y1,
                             int X2, int Y2, int MODO);
int      ORBISAPI GetBMPSettings (LPSTR Arquivo, LPINT TamX, LPINT TamY);
int      ORBISAPI ShowTIFF (LPSTR Arquivo, int X1, int Y1,
                             int X2, int Y2, int MODO);
int      ORBISAPI GetTIFFSettings (LPSTR Arquivo, LPINT TamX, LPINT TamY);
int      ORBISAPI BuildPCX (LPSTR Arquivo, int X1, int Y1,
                             int X2, int Y2, int PLANOS);
void     ORBISAPI LimpaTelaGrafica (int Cor);

void     ORBISAPI DescarregaFonte (void);
int      ORBISAPI CarregaFonte (LPSTR lpszNomeArqFonte);
void     ORBISAPI DesenhaCaractereFonte (int Cod);
void     ORBISAPI sayf (int X, int Y, LPSTR lpszCadeia);
int      ORBISAPI ExtensaoTexto (LPSTR lpszCadeia);
#endif

#ifndef SEM_COMUNICACAO
/***** Rotinas de comunicao ******/
void     ORBISAPI _RS232_PRG   (int PORTA, LPSTR SETUP, LPSTR DIAL, LPSTR CANCEL);
int      ORBISAPI _RS232_CFG   (int PORTA, long BAUDS, int PARID, int STOP, int BITS);      // 4.0
int      ORBISAPI _RS232_HWCFG (int iPorta, int iIOadd, int iIrq, int iDSR, int iCTS, int iDCD);    // 4.0
int      ORBISAPI _RS232_CONECTADO (int iPorta);    // 4.0
int      ORBISAPI _RS232_ENA   (int PORTA);
int      ORBISAPI _RS232_DIS   (int PORTA);
int      ORBISAPI _RS232_DIAL  (int PORTA, LPSTR NUMERO, int iEspera);
void     ORBISAPI _RS232_ABT   (int PORTA);
int      ORBISAPI _RS232_TX    (int PORTA, LPSTR CADEIA, int DELAY, int TAMANHO);
int      ORBISAPI _RS232_TXC   (int iPorta, int iDado);
int      ORBISAPI _RS232_RX    (int PORTA, LPSTR CADEIA, int ECO, int TEMPO, char TERMIN, int TAMANHO);
int      ORBISAPI _RS232_RXC   (int PORTA, int ECO, int TEMPO);
int      ORBISAPI _RS232_BRK   (int PORTA);
unsigned ORBISAPI _RS232_STAT  (int PORTA);
int      ORBISAPI EnviaXMODEM  (int iPorta, LPSTR lpszArq);
int      ORBISAPI EnviaYMODEM  (int iPorta, LPSTR lpszMascara);
int      ORBISAPI RecebeXMODEM (int iPorta, LPSTR lpszArq);
int      ORBISAPI RecebeYMODEM (int iPorta, LPSTR lpszDir);
#endif

#ifndef SEM_DOS
/****** Interface com o DOS ******/
#define MT_NONE     0
#define MT_DESQVIEW 1
#define MT_WIN_3    2
#define MT_OS2_20   3
#define MT_OS2_21   4

long     ORBISAPI QualCPU (void);
int      ORBISAPI RAMtotal (void);
int      ORBISAPI XMEMtotal (void);
int      ORBISAPI EMStotal (void);
int      ORBISAPI EMSlivre (void);
long     ORBISAPI Tipodrv (char Drive);
int      ORBISAPI DrvLocal (char Drive);
int      ORBISAPI DrvFixo (char Drive);
int      ORBISAPI DrvReal (char Drive);
int      ORBISAPI NPU (void);
int      ORBISAPI COMS (void);
int      ORBISAPI GAMEP (void);
int      ORBISAPI LPTS (void);
int      ORBISAPI XMEMlivre (void);
int      ORBISAPI FILESLFT (void);
void     ORBISAPI Wboot (void);
void     ORBISAPI Boot (void);
long     ORBISAPI KbdStat (void);
long     ORBISAPI VolSerie (char Drive);
int      ORBISAPI DOSver (void);
int      ORBISAPI FILES (void);
int      ORBISAPI BUFFERS (void);
LPSTR    ORBISAPI FileName (int Handle);
LPSTR    ORBISAPI RealDir (char Drive);
LPSTR    ORBISAPI Truename (LPSTR lpszNomarq);
LPSTR    ORBISAPI VolLabel (char Drive);
int      ORBISAPI swap (LPSTR program_name,
                          LPSTR command_line,
                          LPSTR exec_return,
                          LPSTR swap_fname); // 3.0
int      ORBISAPI do_exec      (LPSTR exfn, LPSTR epars); // 3.0
int      ORBISAPI QualMultitarefa (void);    // 4.0
void     ORBISAPI LiberaTimeslice (void);    // 4.0
int      ORBISAPI IsCDROMDrive (char cDrive);    // 4.0
int      ORBISAPI IsDriveReady (char cDrive);    // 4.0
void     ORBISAPI _xy (int x, int y);            // 4.0
void     ORBISAPI _putstr (int x, int y, int cor, LPSTR lpszStr);    // 4.0
#endif

#ifndef SEM_IMPRESSAO
typedef enum
   {
   EPSON=1,
   HPLJ2=2,
   HPDJ500=3
   } TIPO_IMPRESSORA;

/***** Rotinas de Impressora *****/
void     ORBISAPI PrSc (int impressora,
                        int afast,
                        TIPO_IMPRESSORA tipo,
                        int escala_h,
                        int escala_v,
                        int densidade,
                        int reverso);
int      ORBISAPI LeFntHP  (LPSTR lpszFonte);
LPSTR    ORBISAPI SeleFnt (void);
int      ORBISAPI MedeLin  (LPSTR lpszCadeia, int iTamCadeia);
void     ORBISAPI AcImp    (int IMP);
#endif

#ifndef SEM_COMPRESSAO
/**** Rotinas de Compresso de Dados ********/
int      ORBISAPI CompactaArquivo (LPSTR lpszNomeNormal,
                                     LPSTR lpszNomeCompactado,
                                     WORD16 uiPassword);

int      ORBISAPI DescompactaArquivo (LPSTR lpszNomeCompactado,
                                        LPSTR lpszNomeNormal,
                                        WORD16 uiPassword);

int      ORBISAPI CompactaBuffer (LPBYTE lpBufNorm,
                                    int nTamBufNorm,
                                    LPBYTE lpBufPacked,
                                    int nTamBufPacked,
                                    WORD16 uiPassword);

int      ORBISAPI DescompactaBuffer (LPBYTE lpBufPacked,
                                       int nTamBufPacked,
                                       LPBYTE lpBufNorm,
                                       int nTamBufNorm,
                                       WORD16 uiPassword);

int      ORBISAPI EncodeLZW    (LPSTR lpszNormal, LPSTR lpszLZW);  // 3.0
int      ORBISAPI DecodeLZW    (LPSTR lpszLZW, LPSTR lpszNormal);  // 3.0
#endif

#ifndef SEM_TELA
/**** Rotinas de Tratamento de tela *****/
void     ORBISAPI  _ASMPOPW   (int, int, int, int, int);
void     ORBISAPI  _ASMFRENTE (int, int, int, int, int);
void     ORBISAPI  _ASMFUNDO  (int, int, int, int, int);
#endif

#ifndef SEM_ACENTUACAO
/**** Rotinas de Acentuao *****/
void     ORBISAPI _INSTALA_KEYACENT    (int CODEPAGE);
void     ORBISAPI _INSTALA_IMPACENT    (int IMP, int CODEPAGE);
void     ORBISAPI _DESINSTALA_KEYACENT (void);
void     ORBISAPI _DESINSTALA_IMPACENT (void);
void     ORBISAPI _CHCP                (int CODEPAGE);
#endif

#ifndef SEM_MOUSE
/**** Rotinas de suporte ao Mouse ****/
int      ORBISAPI ASM_MOU_INIT  (int MODO);
void     ORBISAPI ASM_MOU_END   (void);
int      ORBISAPI ASM_MOU_BUTTS (void);
int      ORBISAPI ASM_MOU_X     (void);
int      ORBISAPI ASM_MOU_Y     (void);
void     ORBISAPI ASM_MOU_SHOW  (void);
void     ORBISAPI ASM_MOU_HIDE  (void);
void     ORBISAPI ASM_MOU_SETP  (int LIN, int COL);
int      ORBISAPI ASM_MOU_GETC  (void);
#endif

#ifndef SEM_XMS
typedef unsigned  XMSHND;
typedef struct
   {
   WORD32   dwBytes;
   XMSHND   hSrc;
   WORD32   dwOffSrc;
   XMSHND   hDst;
   WORD32   dwOffDst;
   } EMM;

typedef WORD32 (ORBISAPI *LPXMSDRIVERPROC) (WORD32);

/***** Rotinas de suporte  memria expandida (XMS) ****/
LPXMSDRIVERPROC     GetXMSDriverAddress (void);
int      ORBISAPI IsXMSDriverPresent (void);
XMSHND   ORBISAPI AllocXMSBlock (WORD16 wSize);
int      ORBISAPI FreeXMSBlock (XMSHND hHandle);
int      ORBISAPI WriteXMS (XMSHND hHandle, LPVOID lpBuffer, WORD16 wSize);
int      ORBISAPI ReadXMS   (XMSHND hHandle, LPVOID lpBuffer, WORD16 wSize);
#endif

#ifndef SEM_MULTIMIDIA
/********* Multimidia ********************/
int      ORBISAPI oplayFLx     (LPSTR nome, int, int);             // 3.0
int      ORBISAPI playWAV      (LPSTR nome);                       // 3.0
int      ORBISAPI recWAV       (LPSTR nome, long lSamplingRate);   // 3.0
#endif

#ifndef SEM_REDE
/******** Rede ***************/
int      ORBISAPI Novell       (void);         // 3.0
void     ORBISAPI NomeNW       (LPSTR Nome);   // 3.0
int      ORBISAPI LanMan       (void);         // 3.0
void     ORBISAPI NomeLM       (LPSTR Nome);   // 3.0
#endif


#if defined ( __cplusplus )
}
#endif  /* __cplusplus */

#pragma pack ()  /* Restaura alinhamento original */

#endif
